﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SqlServer.Dts.Runtime;
using SqlBI.DtExec.Classes;

namespace SqlBI.DtExec.Services {
    /// <summary>
    /// 
    ///     This class intercepts events raised from inside the SSIS
    ///     package and sends them back to the client
    /// 
    /// </summary>
    public class EventHandler : DefaultEvents {
        private ServiceSqlBIDtExec Caller = null;
        private EventsOption EventOptions;

        /// <summary>
        /// 
        ///     Constructor for the class, it receives the caller as a parameter
        ///     in order to be able to invoke the callback method
        /// 
        /// </summary>
        /// <param name="caller"></param>
        public EventHandler (ServiceSqlBIDtExec caller, EventsOption eventOptions) {
            Caller = caller;
            EventOptions = eventOptions;
        }

        /// <summary>
        /// 
        ///     Restituisce il nome completo di un componente
        /// 
        /// </summary>
        /// <param name="eventsProvider"></param>
        /// <returns></returns>
        private string GetNameFor (EventsProvider eventsProvider) {
            List<string> names = new List<string> ();
            string result = "";
            EventsProvider parentCursor = eventsProvider;
            do {
                names.Add (parentCursor.Name);
                parentCursor = (EventsProvider) parentCursor.Parent;
            } while (parentCursor != null);
            for (int i = names.Count - 1; i >= 0; i--) {
                result += names[i] + @"\";
            }
            return result.Substring (0, result.Length - 1);
        }

        /// <summary>
        /// 
        ///     Logs the Pre-Execute action
        /// 
        /// </summary>
        /// <param name="exec"></param>
        /// <param name="fireAgain"></param>
        public override void OnPreExecute (Executable exec, ref bool fireAgain) {
            base.OnPreExecute (exec, ref fireAgain);

            if (EventOptions.ReportPreExecute) {
                if (exec is EventsProvider) {
                    LogEventsProviderEvent (exec as EventsProvider, "PreExecute");
                }
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        ///     Fires when the package sends an information packet
        /// 
        /// </summary>
        /// <param name="source"></param>
        /// <param name="informationCode"></param>
        /// <param name="subComponent"></param>
        /// <param name="description"></param>
        /// <param name="helpFile"></param>
        /// <param name="helpContext"></param>
        /// <param name="idofInterfaceWithError"></param>
        /// <param name="fireAgain"></param>
        public override void OnInformation (DtsObject source, int informationCode, string subComponent, string description, string helpFile, int helpContext, string idofInterfaceWithError, ref bool fireAgain) {
            base.OnInformation (source, informationCode, subComponent, description, helpFile, helpContext, idofInterfaceWithError, ref fireAgain);

            if (EventOptions.ReportInformation) {
                if (source is EventsProvider) {
                    LogEventsProviderEvent (source as EventsProvider, description);
                }
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="source"></param>
        /// <param name="errorCode"></param>
        /// <param name="subComponent"></param>
        /// <param name="description"></param>
        /// <param name="helpFile"></param>
        /// <param name="helpContext"></param>
        /// <param name="idofInterfaceWithError"></param>
        /// <returns></returns>
        public override bool OnError (DtsObject source, int errorCode, string subComponent, string description, string helpFile, int helpContext, string idofInterfaceWithError) {
            bool result = base.OnError (source, errorCode, subComponent, description, helpFile, helpContext, idofInterfaceWithError);
            if (EventOptions.ReportErrors) {
                if (source is EventsProvider) {
                    LogEventsProviderEvent ((source as EventsProvider), String.Format ("Error: {0} 0x{1} : {2}", subComponent, errorCode.ToString ("X"), description));
                }
            }
            return result;
        }

        /// <summary>
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="exec"></param>
        /// <param name="fireAgain"></param>
        public override void OnPostValidate (Executable exec, ref bool fireAgain) {
            base.OnPostValidate (exec, ref fireAgain);
            if (EventOptions.ReportPostValidate) {
                if (exec is EventsProvider) {
                    LogEventsProviderEvent (exec as EventsProvider, "post validate");
                }
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="exec"></param>
        /// <param name="fireAgain"></param>
        public override void OnPreValidate (Executable exec, ref bool fireAgain) {
            base.OnPreValidate (exec, ref fireAgain);
            if (EventOptions.ReportPreValidate) {
                if (exec is EventsProvider) {
                    LogEventsProviderEvent (exec as EventsProvider, "post validate");
                }
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        /// 
        /// 
        /// </summary>
        /// <param name="DtsContainer"></param>
        /// <param name="variable"></param>
        /// <param name="fireAgain"></param>
        public override void OnVariableValueChanged (DtsContainer DtsContainer, Variable variable, ref bool fireAgain) {
            if (EventOptions.ReportVariableValueChanged) {
                base.OnVariableValueChanged (DtsContainer, variable, ref fireAgain);
                if (DtsContainer is EventsProvider) {
                    string variableValue = "";
                    if (variable.Value == null) {
                        variableValue = "(NULL)";
                    } else {
                        variableValue = variable.Value.ToString ();
                    }

                    LogEventsProviderEvent (DtsContainer as EventsProvider, String.Format ("Variable {0} changed value to {1}", variable.Name, variableValue));
                }
            } else {
                fireAgain = false;
            }

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="taskHost"></param>
        /// <param name="eventName"></param>
        /// <param name="eventText"></param>
        /// <param name="arguments"></param>
        /// <param name="subComponent"></param>
        /// <param name="fireAgain"></param>
        public override void OnCustomEvent (TaskHost taskHost, string eventName, string eventText, ref object[] arguments, string subComponent, ref bool fireAgain) {
            if (EventOptions.ReportCustomEvents) {
                base.OnCustomEvent (taskHost, eventName, eventText, ref arguments, subComponent, ref fireAgain);
                LogEventsProviderEvent (taskHost, String.Format ("Custom Event: {0} - {1}", eventName, eventText));
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        ///     Reports a TASK FAILED event
        /// 
        /// </summary>
        /// <param name="taskHost"></param>
        public override void OnTaskFailed (TaskHost taskHost) {
            if (EventOptions.ReportTaskFailed) {
                base.OnTaskFailed (taskHost);
                LogEventsProviderEvent (taskHost, "Task FAILED");
            }
        }

        /// <summary>
        /// 
        ///     
        /// 
        /// </summary>
        /// <param name="taskHost"></param>
        /// <param name="progressDescription"></param>
        /// <param name="percentComplete"></param>
        /// <param name="progressCountLow"></param>
        /// <param name="progressCountHigh"></param>
        /// <param name="subComponent"></param>
        /// <param name="fireAgain"></param>
        public override void OnProgress (TaskHost taskHost, string progressDescription, int percentComplete, int progressCountLow, int progressCountHigh, string subComponent, ref bool fireAgain) {
            if (EventOptions.ReportProgress) {
                base.OnProgress (taskHost, progressDescription, percentComplete, progressCountLow, progressCountHigh, subComponent, ref fireAgain);
                LogEventsProviderEvent (taskHost, String.Format ("Progress: {0} - {1}% complete.", progressDescription, percentComplete));
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        ///     Post-execution of a component
        /// 
        /// </summary>
        /// <param name="exec"></param>
        /// <param name="fireAgain"></param>
        public override void OnPostExecute (Executable exec, ref bool fireAgain) {
            base.OnPostExecute (exec, ref fireAgain);
            if (EventOptions.ReportInformation) {
                if (exec is EventsProvider) {
                    LogEventsProviderEvent (exec as EventsProvider, "post execution");
                }
            } else {
                fireAgain = false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="source"></param>
        /// <param name="warningCode"></param>
        /// <param name="subComponent"></param>
        /// <param name="description"></param>
        /// <param name="helpFile"></param>
        /// <param name="helpContext"></param>
        /// <param name="idofInterfaceWithError"></param>
        public override void OnWarning (DtsObject source, int warningCode, string subComponent, string description, string helpFile, int helpContext, string idofInterfaceWithError) {
            base.OnWarning (source, warningCode, subComponent, description, helpFile, helpContext, idofInterfaceWithError);
            if (EventOptions.ReportWarnings) {
                if (source is EventsProvider) {
                    LogEventsProviderEvent ((source as EventsProvider), String.Format ("Warning: {0} 0x{1} : {2}", subComponent, warningCode.ToString ("X"), description));
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="exec"></param>
        /// <param name="eventDescription"></param>
        private void LogEventsProviderEvent (EventsProvider exec, string eventDescription) {
            Caller.SendBackMessage (LogType.Event, String.Format ("{0} {1}", GetNameFor (exec), eventDescription));
        }

        /// <summary>
        /// 
        ///     Sends the package start log
        /// 
        /// </summary>
        /// <param name="package"></param>
        private void LogPackageStart (Package package) {
            Caller.SendBackMessage (LogType.Event, String.Format ("Package {0} started.", package.Name));
            Caller.SendBackMessage (LogType.Event, "Connection Managers used in package:");
            foreach (ConnectionManager cm in package.Connections) {
                Caller.SendBackMessage (LogType.Event, String.Format ("\t{0}: {1}", cm.Name, cm.ConnectionString));
            }
        }
    }
}
